# Generated by Django 4.2.25 on 2025-10-17 05:40

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "taxsystem",
            "0008_journalfilter_journalfilterset_delete_filteramount_and_more",
        ),
    ]

    operations = [
        migrations.AddField(
            model_name="payments",
            name="corporation_id",
            field=models.IntegerField(blank=True, null=True),
        ),
    ]
