# Generated by Django 4.2.26 on 2025-11-22 21:54

# Django
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("taxsystem", "0017_alter_allianceupdatestatus_unique_together_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="allianceowner",
            name="corporation",
            field=models.ForeignKey(
                help_text="The corporation that owns this alliance tax system.",
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="taxsystem.corporationowner",
            ),
        ),
    ]
