# Generated by Django 4.2.28 on 2026-02-25 19:32

# Django
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("taxsystem", "0024_alter_allianceadminhistory_action_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="EveEntity",
            fields=[
                ("id", models.BigIntegerField(primary_key=True, serialize=False)),
                ("name", models.CharField(max_length=255)),
                ("category", models.CharField(max_length=50)),
                ("last_updated", models.DateTimeField(auto_now=True)),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.AddField(
            model_name="corporationwalletjournalentry",
            name="first_party_new",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.SET_DEFAULT,
                related_name="+",
                to="taxsystem.eveentity",
            ),
        ),
        migrations.AddField(
            model_name="corporationwalletjournalentry",
            name="second_party_new",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.SET_DEFAULT,
                related_name="+",
                to="taxsystem.eveentity",
            ),
        ),
    ]
