# Generated by Django 4.2.26 on 2025-11-20 13:03

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("taxsystem", "0015_alter_general_options"),
    ]

    operations = [
        migrations.AlterField(
            model_name="allianceupdatestatus",
            name="section",
            field=models.CharField(
                choices=[
                    ("payment_system", "Payment System"),
                    ("payments", "Payments"),
                    ("payday", "Payday"),
                ],
                db_index=True,
                max_length=32,
            ),
        ),
        migrations.AlterField(
            model_name="corporationupdatestatus",
            name="section",
            field=models.CharField(
                choices=[
                    ("division_names", "Wallet Division Names"),
                    ("division", "Wallet Division"),
                    ("wallet", "Wallet Journal"),
                    ("members", "Members"),
                    ("payment_system", "Payment System"),
                    ("payments", "Payments"),
                    ("payday", "Payday"),
                ],
                db_index=True,
                max_length=32,
            ),
        ),
        migrations.RemoveField(
            model_name="corporationpayments",
            name="corporation_id",
        ),
        migrations.AddField(
            model_name="corporationpayments",
            name="owner_id",
            field=models.PositiveIntegerField(
                blank=True,
                help_text="ID of the owner (corporation or alliance) associated with this payment",
                null=True,
            ),
        ),
        migrations.AddIndex(
            model_name="alliancepayments",
            index=models.Index(
                fields=["account", "owner_id", "request_status", "-date"],
                name="taxsystem_a_account_677604_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="alliancepayments",
            index=models.Index(
                fields=["request_status", "-date"],
                name="taxsystem_a_request_d243c8_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="corporationpayments",
            index=models.Index(
                fields=["account", "owner_id", "request_status", "-date"],
                name="taxsystem_c_account_dafabb_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="corporationpayments",
            index=models.Index(
                fields=["request_status", "-date"],
                name="taxsystem_c_request_5fb377_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="members",
            index=models.Index(
                fields=["owner", "character_name"],
                name="taxsystem_m_owner_i_f6738f_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="members",
            index=models.Index(fields=["status"], name="taxsystem_m_status_38e108_idx"),
        ),
    ]
