# Generated by Django 4.2.26 on 2026-01-11 12:42

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("taxsystem", "0023_alliancefilter_match_type_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="allianceadminhistory",
            name="action",
            field=models.CharField(
                choices=[
                    ("", ""),
                    ("Added", "Added"),
                    ("Changed", "Changed"),
                    ("Deleted", "Deleted"),
                ],
                default="",
                help_text="Action performed",
                max_length=20,
                verbose_name="Action",
            ),
        ),
        migrations.AlterField(
            model_name="alliancepaymentaccount",
            name="status",
            field=models.CharField(
                blank=True,
                choices=[
                    ("active", "Active"),
                    ("inactive", "Inactive"),
                    ("deactivated", "Deactivated"),
                    ("missing", "Missing"),
                ],
                default="active",
                max_length=16,
            ),
        ),
        migrations.AlterField(
            model_name="alliancepaymenthistory",
            name="new_status",
            field=models.CharField(
                choices=[
                    ("approved", "Approved"),
                    ("pending", "Pending"),
                    ("rejected", "Rejected"),
                    ("needs_approval", "Requires Auditor"),
                ],
                help_text="New Status of the action",
                max_length=20,
                verbose_name="New Status",
            ),
        ),
        migrations.AlterField(
            model_name="alliancepayments",
            name="request_status",
            field=models.CharField(
                choices=[
                    ("approved", "Approved"),
                    ("pending", "Pending"),
                    ("rejected", "Rejected"),
                    ("needs_approval", "Requires Auditor"),
                ],
                default="pending",
                max_length=16,
                verbose_name="Request Status",
            ),
        ),
        migrations.AlterField(
            model_name="corporationadminhistory",
            name="action",
            field=models.CharField(
                choices=[
                    ("", ""),
                    ("Added", "Added"),
                    ("Changed", "Changed"),
                    ("Deleted", "Deleted"),
                ],
                default="",
                help_text="Action performed",
                max_length=20,
                verbose_name="Action",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpaymentaccount",
            name="status",
            field=models.CharField(
                blank=True,
                choices=[
                    ("active", "Active"),
                    ("inactive", "Inactive"),
                    ("deactivated", "Deactivated"),
                    ("missing", "Missing"),
                ],
                default="active",
                max_length=16,
            ),
        ),
        migrations.AlterField(
            model_name="corporationpaymenthistory",
            name="new_status",
            field=models.CharField(
                choices=[
                    ("approved", "Approved"),
                    ("pending", "Pending"),
                    ("rejected", "Rejected"),
                    ("needs_approval", "Requires Auditor"),
                ],
                help_text="New Status of the action",
                max_length=20,
                verbose_name="New Status",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpayments",
            name="request_status",
            field=models.CharField(
                choices=[
                    ("approved", "Approved"),
                    ("pending", "Pending"),
                    ("rejected", "Rejected"),
                    ("needs_approval", "Requires Auditor"),
                ],
                default="pending",
                max_length=16,
                verbose_name="Request Status",
            ),
        ),
        migrations.AlterField(
            model_name="members",
            name="status",
            field=models.CharField(
                blank=True,
                choices=[
                    ("active", "Active"),
                    ("missing", "Missing"),
                    ("noaccount", "Unregistered"),
                    ("is_alt", "is alt"),
                ],
                default="active",
                max_length=10,
                verbose_name="Status",
            ),
        ),
    ]
