# Generated by Django 4.2.28 on 2026-02-26 07:21

# Django
import django.core.validators
import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    replaces = [
        ("taxsystem", "0001_initial"),
        ("taxsystem", "0002_alter_paymenthistory_comment_and_more"),
        ("taxsystem", "0003_alter_general_options_alter_paymenthistory_comment"),
        ("taxsystem", "0004_owneraudit_last_update_payday_and_more"),
        (
            "taxsystem",
            "0005_rename_division_corporationwalletdivision_division_id_and_more",
        ),
        ("taxsystem", "0006_create_update_status_existings"),
        ("taxsystem", "0007_alter_ownerupdatestatus_section"),
        (
            "taxsystem",
            "0008_journalfilter_journalfilterset_delete_filteramount_and_more",
        ),
        ("taxsystem", "0009_payments_corporation_id"),
        ("taxsystem", "0010_create_alliance_corporation"),
        ("taxsystem", "0011_alter_corporationowner_options_and_more"),
        ("taxsystem", "0012_alliancefilter_alliancefilterset_and_more"),
        ("taxsystem", "0013_rename_adminlogs_corporationadminhistory_and_more"),
        (
            "taxsystem",
            "0014_rename_allianceownerupdatestatus_allianceupdatestatus_and_more",
        ),
        ("taxsystem", "0015_alter_general_options"),
        ("taxsystem", "0016_remove_corporationpayments_corporation_id_and_more"),
        ("taxsystem", "0017_alter_allianceupdatestatus_unique_together_and_more"),
        ("taxsystem", "0018_alter_allianceowner_corporation"),
        ("taxsystem", "0019_alter_alliancepayments_entry_id_and_more"),
        ("taxsystem", "0020_alter_general_options_and_more"),
        (
            "taxsystem",
            "0021_remove_alliancepayments_taxsystem_a_account_677604_idx_and_more",
        ),
        ("taxsystem", "0022_alter_alliancepaymenthistory_new_status_and_more"),
        ("taxsystem", "0023_alliancefilter_match_type_and_more"),
        ("taxsystem", "0024_alter_allianceadminhistory_action_and_more"),
        ("taxsystem", "0025_eveentity_and_more"),
        (
            "taxsystem",
            "0026_remove_corporationwalletjournalentry_taxsystem_c_first_p_d83db8_idx_and_more",
        ),
        (
            "taxsystem",
            "0027_rename_first_party_new_corporationwalletjournalentry_first_party_and_more",
        ),
    ]

    initial = True

    dependencies = [
        ("eveonline", "0017_alliance_and_corp_names_are_not_unique"),
        ("contenttypes", "0002_remove_content_type_name"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="OwnerAudit",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=255)),
                ("active", models.BooleanField(default=True)),
                (
                    "tax_amount",
                    models.DecimalField(
                        decimal_places=0,
                        default=0,
                        help_text="Tax Amount in ISK that is set for the corporation. Max 16 Digits",
                        max_digits=16,
                        validators=[
                            django.core.validators.MaxValueValidator(9999999999999999)
                        ],
                    ),
                ),
                (
                    "tax_period",
                    models.PositiveIntegerField(
                        default=30,
                        help_text="Tax Period in days for the corporation. Max 365 days. Default: 30 days",
                        validators=[django.core.validators.MaxValueValidator(365)],
                    ),
                ),
                (
                    "alliance",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="alliance",
                        to="eveonline.eveallianceinfo",
                    ),
                ),
                (
                    "corporation",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="eveonline.evecorporationinfo",
                    ),
                ),
            ],
            options={
                "verbose_name": "Tax System Audit",
                "verbose_name_plural": "Tax System Audits",
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="EveEntity",
            fields=[
                ("id", models.BigIntegerField(primary_key=True, serialize=False)),
                ("name", models.CharField(max_length=255)),
                ("category", models.CharField(max_length=50)),
                ("last_updated", models.DateTimeField(auto_now=True)),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="PaymentSystem",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=100)),
                ("date", models.DateTimeField(auto_now_add=True, null=True)),
                (
                    "status",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("active", "Active"),
                            ("inactive", "Inactive"),
                            ("deactivated", "Deactivated"),
                            ("missing", "Missing"),
                        ],
                        default="active",
                        max_length=16,
                    ),
                ),
                (
                    "deposit",
                    models.DecimalField(
                        decimal_places=0,
                        default=0,
                        help_text="Deposit Pool in ISK. Max 16 Digits",
                        max_digits=16,
                        validators=[
                            django.core.validators.MaxValueValidator(9999999999999999),
                            django.core.validators.MinValueValidator(-9999999999999999),
                        ],
                    ),
                ),
                ("last_paid", models.DateTimeField(blank=True, null=True)),
                ("notice", models.TextField(blank=True, null=True)),
                (
                    "owner",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="ts_payment_system",
                        to="taxsystem.owneraudit",
                    ),
                ),
                (
                    "user",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "Tax Payment System",
                "verbose_name_plural": "Tax Payment Systems",
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="Payments",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=100)),
                ("entry_id", models.BigIntegerField(blank=True, null=True)),
                ("amount", models.DecimalField(decimal_places=0, max_digits=12)),
                ("date", models.DateTimeField(blank=True, null=True)),
                ("reason", models.TextField(blank=True, null=True)),
                (
                    "request_status",
                    models.CharField(
                        choices=[
                            ("approved", "Approved"),
                            ("pending", "Pending"),
                            ("rejected", "Rejected"),
                            ("needs_approval", "Requires Auditor"),
                        ],
                        default="pending",
                        max_length=16,
                        verbose_name="Request Status",
                    ),
                ),
                (
                    "reviser",
                    models.CharField(
                        blank=True,
                        default="",
                        help_text="Reviser that approved or rejected the payment",
                        max_length=100,
                    ),
                ),
                (
                    "account",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="ts_payments",
                        to="taxsystem.paymentsystem",
                    ),
                ),
            ],
            options={
                "verbose_name": "Tax Payments",
                "verbose_name_plural": "Tax Payments",
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="CorporationWalletDivision",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(default=None, max_length=100, null=True)),
                ("balance", models.DecimalField(decimal_places=2, max_digits=20)),
                ("division_id", models.IntegerField()),
                (
                    "corporation",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="ts_corporation_division",
                        to="taxsystem.owneraudit",
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="General",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
            options={
                "permissions": (
                    ("basic_access", "Can access the Tax System"),
                    ("create_access", "Can add Corporation"),
                    ("manage_own_corp", "Can manage own Corporation"),
                    ("manage_corps", "Can manage all Corporations"),
                ),
                "managed": False,
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="PaymentHistory",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "date",
                    models.DateTimeField(
                        default=django.utils.timezone.now,
                        help_text="Date of the action",
                        verbose_name="Date",
                    ),
                ),
                (
                    "action",
                    models.CharField(
                        choices=[
                            ("", ""),
                            ("Status Changed", "Status Changed"),
                            ("Payment Added", "Payment Added"),
                            ("Reviser Comment", "Reviser Comment"),
                        ],
                        default="",
                        help_text="Action performed",
                        max_length=20,
                        verbose_name="Action",
                    ),
                ),
                (
                    "comment",
                    models.CharField(
                        choices=[
                            ("", ""),
                            ("Payment added to system", "Payment added to system"),
                            (
                                "Automated approved Payment",
                                "Automated approved Payment",
                            ),
                            (
                                "Payment must be approved by an reviser",
                                "Payment must be approved by an reviser",
                            ),
                        ],
                        default="",
                        help_text="Comment of the action",
                        max_length=255,
                        verbose_name="Comment",
                    ),
                ),
                (
                    "new_status",
                    models.CharField(
                        choices=[
                            ("approved", "Approved"),
                            ("pending", "Pending"),
                            ("rejected", "Rejected"),
                            ("needs_approval", "Requires Auditor"),
                        ],
                        help_text="New Status of the action",
                        max_length=16,
                        verbose_name="New Status",
                    ),
                ),
                (
                    "payment",
                    models.ForeignKey(
                        help_text="Payment that the action was performed on",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="taxsystem.payments",
                        verbose_name="Payment",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        help_text="User that performed the action",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="User",
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="Members",
            fields=[
                ("character_name", models.CharField(db_index=True, max_length=100)),
                (
                    "character_id",
                    models.PositiveIntegerField(primary_key=True, serialize=False),
                ),
                (
                    "status",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("active", "Active"),
                            ("missing", "Missing"),
                            ("noaccount", "Unregistered"),
                            ("is_alt", "Is Alt"),
                        ],
                        default="active",
                        max_length=10,
                        verbose_name="Status",
                    ),
                ),
                ("logon", models.DateTimeField(blank=True, null=True)),
                ("logged_off", models.DateTimeField(blank=True, null=True)),
                ("joined", models.DateTimeField(blank=True, null=True)),
                ("notice", models.TextField(blank=True, null=True)),
                (
                    "owner",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="ts_members",
                        to="taxsystem.owneraudit",
                    ),
                ),
            ],
            options={
                "verbose_name": "Tax Member System",
                "verbose_name_plural": "Tax Member Systems",
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="AdminLogs",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "date",
                    models.DateTimeField(
                        default=django.utils.timezone.now,
                        help_text="Date of the action",
                        verbose_name="Date",
                    ),
                ),
                (
                    "action",
                    models.CharField(
                        choices=[
                            ("", ""),
                            ("Added", "Added"),
                            ("Changed", "Changed"),
                            ("Deleted", "Deleted"),
                        ],
                        default="",
                        help_text="Action performed",
                        max_length=20,
                        verbose_name="Action",
                    ),
                ),
                (
                    "comment",
                    models.TextField(blank=True, default="", verbose_name="Comment"),
                ),
                (
                    "owner",
                    models.ForeignKey(
                        help_text="Payment that the action was performed on",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="taxsystem.owneraudit",
                        verbose_name="Payment",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        help_text="User that performed the action",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="User",
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="CorporationWalletJournalEntry",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "amount",
                    models.DecimalField(
                        decimal_places=2, default=None, max_digits=20, null=True
                    ),
                ),
                (
                    "balance",
                    models.DecimalField(
                        decimal_places=2, default=None, max_digits=20, null=True
                    ),
                ),
                ("context_id", models.BigIntegerField(default=None, null=True)),
                (
                    "context_id_type",
                    models.CharField(
                        choices=[
                            ("structure_id", "Structure Id"),
                            ("station_id", "Station Id"),
                            ("market_transaction_id", "Market Transaction Id"),
                            ("character_id", "Character Id"),
                            ("corporation_id", "Corporation Id"),
                            ("alliance_id", "Alliance Id"),
                            ("eve_system", "Eve System"),
                            ("industry_job_id", "Industry Job Id"),
                            ("contract_id", "Contract Id"),
                            ("planet_id", "Planet Id"),
                            ("system_id", "System Id"),
                            ("type_id", "Type Id"),
                        ],
                        default=None,
                        max_length=30,
                        null=True,
                    ),
                ),
                ("date", models.DateTimeField()),
                ("description", models.CharField(max_length=500)),
                ("entry_id", models.BigIntegerField()),
                ("reason", models.CharField(default=None, max_length=500, null=True)),
                ("ref_type", models.CharField(max_length=72)),
                (
                    "tax",
                    models.DecimalField(
                        decimal_places=2, default=None, max_digits=20, null=True
                    ),
                ),
                ("tax_receiver_id", models.IntegerField(default=None, null=True)),
                (
                    "division",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="ts_corporation_wallet",
                        to="taxsystem.corporationwalletdivision",
                    ),
                ),
                (
                    "first_party",
                    models.ForeignKey(
                        blank=True,
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.SET_DEFAULT,
                        related_name="+",
                        to="taxsystem.eveentity",
                    ),
                ),
                (
                    "second_party",
                    models.ForeignKey(
                        blank=True,
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.SET_DEFAULT,
                        related_name="+",
                        to="taxsystem.eveentity",
                    ),
                ),
            ],
            options={
                "abstract": False,
                "default_permissions": (),
                "indexes": [
                    models.Index(fields=["date"], name="taxsystem_c_date_1a3f46_idx"),
                    models.Index(
                        fields=["amount"], name="taxsystem_c_amount_3f4bd8_idx"
                    ),
                    models.Index(
                        fields=["entry_id"], name="taxsystem_c_entry_i_bd9090_idx"
                    ),
                    models.Index(
                        fields=["ref_type"], name="taxsystem_c_ref_typ_5f82e6_idx"
                    ),
                    models.Index(
                        fields=["first_party"], name="taxsystem_c_first_p_d83db8_idx"
                    ),
                    models.Index(
                        fields=["second_party"], name="taxsystem_c_second__2a759e_idx"
                    ),
                ],
            },
        ),
        migrations.CreateModel(
            name="OwnerUpdateStatus",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "section",
                    models.CharField(
                        choices=[
                            ("wallet", "Wallet Journal"),
                            ("division_names", "Wallet Division Names"),
                            ("division", "Wallet Division"),
                            ("members", "Members"),
                            ("payments", "Payments"),
                            ("payment_system", "Payment System"),
                            ("payday", "Payday"),
                        ],
                        db_index=True,
                        max_length=32,
                    ),
                ),
                (
                    "is_success",
                    models.BooleanField(db_index=True, default=None, null=True),
                ),
                ("error_message", models.TextField()),
                ("has_token_error", models.BooleanField(default=False)),
                (
                    "last_run_at",
                    models.DateTimeField(
                        db_index=True,
                        default=None,
                        help_text="Last run has been started at this time",
                        null=True,
                    ),
                ),
                (
                    "last_run_finished_at",
                    models.DateTimeField(
                        db_index=True,
                        default=None,
                        help_text="Last run has been successful finished at this time",
                        null=True,
                    ),
                ),
                (
                    "last_update_at",
                    models.DateTimeField(
                        db_index=True,
                        default=None,
                        help_text="Last update has been started at this time",
                        null=True,
                    ),
                ),
                (
                    "last_update_finished_at",
                    models.DateTimeField(
                        db_index=True,
                        default=None,
                        help_text="Last update has been successful finished at this time",
                        null=True,
                    ),
                ),
                (
                    "owner",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="ts_update_status",
                        to="taxsystem.owneraudit",
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="JournalFilterSet",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=100, unique=True)),
                ("description", models.CharField(blank=True, max_length=255)),
                ("enabled", models.BooleanField(default=True)),
                (
                    "owner",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="ts_filter_set",
                        to="taxsystem.owneraudit",
                    ),
                ),
            ],
            options={
                "verbose_name": "Journal Filter Set",
                "verbose_name_plural": "Journal Filter Sets",
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="JournalFilter",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "filter_type",
                    models.CharField(
                        choices=[("reason", "Reason"), ("amount", "Amount")],
                        max_length=20,
                    ),
                ),
                ("value", models.CharField(max_length=255)),
                (
                    "filter_set",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="ts_filters",
                        to="taxsystem.journalfilterset",
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.AddConstraint(
            model_name="journalfilter",
            constraint=models.UniqueConstraint(
                fields=("filter_set", "value"), name="unique_filter_value_per_set"
            ),
        ),
        migrations.AddField(
            model_name="payments",
            name="corporation_id",
            field=models.IntegerField(blank=True, null=True),
        ),
        migrations.RenameModel(
            old_name="OwnerAudit",
            new_name="CorporationOwner",
        ),
        migrations.RenameModel(
            old_name="Payments",
            new_name="CorporationPayments",
        ),
        migrations.RenameModel(
            old_name="PaymentSystem",
            new_name="CorporationPaymentAccount",
        ),
        migrations.RenameModel(
            old_name="OwnerUpdateStatus",
            new_name="CorporationUpdateStatus",
        ),
        migrations.CreateModel(
            name="AllianceOwner",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=255)),
                ("active", models.BooleanField(default=True)),
                (
                    "tax_amount",
                    models.DecimalField(
                        decimal_places=0,
                        default=0,
                        help_text="Tax Amount in ISK that is set for the corporation. Max 16 Digits",
                        max_digits=16,
                        validators=[
                            django.core.validators.MaxValueValidator(9999999999999999)
                        ],
                    ),
                ),
                (
                    "tax_period",
                    models.PositiveIntegerField(
                        default=30,
                        help_text="Tax Period in days for the corporation. Max 365 days. Default: 30 days",
                        validators=[django.core.validators.MaxValueValidator(365)],
                    ),
                ),
                (
                    "eve_alliance",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="eveonline.eveallianceinfo",
                    ),
                ),
                (
                    "corporation",
                    models.ForeignKey(
                        help_text="The corporation that owns this alliance tax system.",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="taxsystem.corporationowner",
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="AllianceOwnerUpdateStatus",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "is_success",
                    models.BooleanField(db_index=True, default=None, null=True),
                ),
                ("error_message", models.TextField()),
                ("has_token_error", models.BooleanField(default=False)),
                (
                    "last_run_at",
                    models.DateTimeField(
                        db_index=True,
                        default=None,
                        help_text="Last run has been started at this time",
                        null=True,
                    ),
                ),
                (
                    "last_run_finished_at",
                    models.DateTimeField(
                        db_index=True,
                        default=None,
                        help_text="Last run has been successful finished at this time",
                        null=True,
                    ),
                ),
                (
                    "last_update_at",
                    models.DateTimeField(
                        db_index=True,
                        default=None,
                        help_text="Last update has been started at this time",
                        null=True,
                    ),
                ),
                (
                    "last_update_finished_at",
                    models.DateTimeField(
                        db_index=True,
                        default=None,
                        help_text="Last update has been successful finished at this time",
                        null=True,
                    ),
                ),
                (
                    "section",
                    models.CharField(
                        choices=[
                            ("payments", "Payments"),
                            ("payment_system", "Payment System"),
                            ("payday", "Payday"),
                        ],
                        db_index=True,
                        max_length=32,
                    ),
                ),
                (
                    "owner",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="ts_alliance_update_status",
                        to="taxsystem.allianceowner",
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="AlliancePaymentAccount",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=100)),
                ("date", models.DateTimeField(auto_now_add=True, null=True)),
                (
                    "status",
                    models.CharField(
                        blank=True,
                        choices=[
                            ("active", "Active"),
                            ("inactive", "Inactive"),
                            ("deactivated", "Deactivated"),
                            ("missing", "Missing"),
                        ],
                        default="active",
                        max_length=16,
                    ),
                ),
                (
                    "deposit",
                    models.DecimalField(
                        decimal_places=0,
                        default=0,
                        help_text="Deposit Pool in ISK. Max 16 Digits",
                        max_digits=16,
                        validators=[
                            django.core.validators.MaxValueValidator(9999999999999999),
                            django.core.validators.MinValueValidator(-9999999999999999),
                        ],
                    ),
                ),
                ("last_paid", models.DateTimeField(blank=True, null=True)),
                ("notice", models.TextField(blank=True, null=True)),
                (
                    "owner",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="ts_alliance_payment_accounts",
                        to="taxsystem.allianceowner",
                    ),
                ),
                (
                    "user",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="AlliancePayments",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=100)),
                ("entry_id", models.BigIntegerField(blank=True, null=True)),
                ("amount", models.DecimalField(decimal_places=0, max_digits=12)),
                ("date", models.DateTimeField(blank=True, null=True)),
                ("reason", models.TextField(blank=True, null=True)),
                (
                    "request_status",
                    models.CharField(
                        choices=[
                            ("approved", "Approved"),
                            ("pending", "Pending"),
                            ("rejected", "Rejected"),
                            ("needs_approval", "Requires Auditor"),
                        ],
                        default="pending",
                        max_length=16,
                        verbose_name="Request Status",
                    ),
                ),
                (
                    "reviser",
                    models.CharField(
                        blank=True,
                        default="",
                        help_text="Reviser that approved or rejected the payment",
                        max_length=100,
                    ),
                ),
                (
                    "account",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="ts_alliance_payments",
                        to="taxsystem.alliancepaymentaccount",
                    ),
                ),
                (
                    "owner_id",
                    models.PositiveIntegerField(
                        blank=True,
                        help_text="ID of the owner (corporation or alliance) associated with this payment",
                        null=True,
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.AlterModelOptions(
            name="corporationowner",
            options={"default_permissions": ()},
        ),
        migrations.AlterModelOptions(
            name="corporationpaymentaccount",
            options={"default_permissions": ()},
        ),
        migrations.AlterModelOptions(
            name="corporationpayments",
            options={"default_permissions": ()},
        ),
        migrations.RenameField(
            model_name="corporationowner",
            old_name="corporation",
            new_name="eve_corporation",
        ),
        migrations.RemoveField(
            model_name="corporationowner",
            name="alliance",
        ),
        migrations.AlterField(
            model_name="corporationpaymentaccount",
            name="owner",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="taxsystem.corporationowner",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpayments",
            name="account",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corp_payments",
                to="taxsystem.corporationpaymentaccount",
            ),
        ),
        migrations.AlterField(
            model_name="corporationupdatestatus",
            name="owner",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corporation_update_status",
                to="taxsystem.corporationowner",
            ),
        ),
        migrations.AlterField(
            model_name="journalfilter",
            name="filter_set",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corporation_filters",
                to="taxsystem.journalfilterset",
            ),
        ),
        migrations.AlterField(
            model_name="journalfilterset",
            name="owner",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corporation_filter_set",
                to="taxsystem.corporationowner",
            ),
        ),
        migrations.CreateModel(
            name="AllianceFilter",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "filter_type",
                    models.CharField(
                        choices=[("reason", "Reason"), ("amount", "Amount")],
                        max_length=20,
                    ),
                ),
                ("value", models.CharField(max_length=255)),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="AllianceFilterSet",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=100, unique=True)),
                ("description", models.CharField(blank=True, max_length=255)),
                ("enabled", models.BooleanField(default=True)),
            ],
            options={
                "abstract": False,
                "default_permissions": (),
            },
        ),
        migrations.RenameModel(
            old_name="JournalFilter",
            new_name="CorporationFilter",
        ),
        migrations.RenameModel(
            old_name="JournalFilterSet",
            new_name="CorporationFilterSet",
        ),
        migrations.AlterModelOptions(
            name="corporationfilterset",
            options={"default_permissions": ()},
        ),
        migrations.AlterModelOptions(
            name="members",
            options={"default_permissions": ()},
        ),
        migrations.RemoveConstraint(
            model_name="corporationfilter",
            name="unique_filter_value_per_set",
        ),
        migrations.AlterField(
            model_name="corporationupdatestatus",
            name="section",
            field=models.CharField(choices=[], db_index=True, max_length=32),
        ),
        migrations.AddField(
            model_name="alliancefilterset",
            name="owner",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_alliance_filter_set",
                to="taxsystem.allianceowner",
            ),
        ),
        migrations.AddField(
            model_name="alliancefilter",
            name="filter_set",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_alliance_filters",
                to="taxsystem.alliancefilterset",
            ),
        ),
        migrations.RenameModel(
            old_name="AdminLogs",
            new_name="CorporationAdminHistory",
        ),
        migrations.RenameModel(
            old_name="PaymentHistory",
            new_name="CorporationPaymentHistory",
        ),
        migrations.AlterField(
            model_name="corporationpaymentaccount",
            name="owner",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corporation_payment_accounts",
                to="taxsystem.corporationowner",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpayments",
            name="account",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corporation_payments",
                to="taxsystem.corporationpaymentaccount",
            ),
        ),
        migrations.AlterField(
            model_name="corporationupdatestatus",
            name="section",
            field=models.CharField(
                choices=[
                    ("wallet", "Wallet Journal"),
                    ("division_names", "Wallet Division Names"),
                    ("division", "Wallet Division"),
                    ("members", "Members"),
                    ("payments", "Payments"),
                    ("payment_system", "Payment System"),
                    ("payday", "Payday"),
                ],
                db_index=True,
                max_length=32,
            ),
        ),
        migrations.RenameModel(
            old_name="AllianceOwnerUpdateStatus",
            new_name="AllianceUpdateStatus",
        ),
        migrations.CreateModel(
            name="AlliancePaymentHistory",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "date",
                    models.DateTimeField(
                        default=django.utils.timezone.now,
                        help_text="Date of the action",
                        verbose_name="Date",
                    ),
                ),
                (
                    "action",
                    models.CharField(
                        choices=[
                            ("", ""),
                            ("Status Changed", "Status Changed"),
                            ("Payment Added", "Payment Added"),
                            ("Reviser Comment", "Reviser Comment"),
                        ],
                        default="",
                        help_text="Action performed",
                        max_length=20,
                        verbose_name="Action",
                    ),
                ),
                (
                    "comment",
                    models.CharField(
                        choices=[
                            ("", ""),
                            ("Payment added to system", "Payment added to system"),
                            (
                                "Automated approved Payment",
                                "Automated approved Payment",
                            ),
                            (
                                "Payment must be approved by an reviser",
                                "Payment must be approved by an reviser",
                            ),
                        ],
                        default="",
                        help_text="Comment of the action",
                        max_length=255,
                        verbose_name="Comment",
                    ),
                ),
                (
                    "new_status",
                    models.CharField(
                        choices=[
                            ("approved", "Approved"),
                            ("pending", "Pending"),
                            ("rejected", "Rejected"),
                            ("needs_approval", "Requires Auditor"),
                        ],
                        help_text="New Status of the action",
                        max_length=16,
                        verbose_name="New Status",
                    ),
                ),
                (
                    "payment",
                    models.ForeignKey(
                        help_text="Payment that the action was performed on",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="taxsystem.alliancepayments",
                        verbose_name="Payment",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        help_text="User that performed the action",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="User",
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="AllianceAdminHistory",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "date",
                    models.DateTimeField(
                        default=django.utils.timezone.now,
                        help_text="Date of the action",
                        verbose_name="Date",
                    ),
                ),
                (
                    "action",
                    models.CharField(
                        choices=[
                            ("", ""),
                            ("Added", "Added"),
                            ("Changed", "Changed"),
                            ("Deleted", "Deleted"),
                        ],
                        default="",
                        help_text="Action performed",
                        max_length=20,
                        verbose_name="Action",
                    ),
                ),
                (
                    "comment",
                    models.TextField(blank=True, default="", verbose_name="Comment"),
                ),
                (
                    "owner",
                    models.ForeignKey(
                        help_text="Payment that the action was performed on",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="taxsystem.allianceowner",
                        verbose_name="Payment",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        help_text="User that performed the action",
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="User",
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
        migrations.AlterModelOptions(
            name="general",
            options={
                "default_permissions": (),
                "managed": False,
                "permissions": (
                    ("basic_access", "Can access the Tax System"),
                    ("create_access", "Can add Corporation"),
                    ("manage_own_corp", "Can manage own Corporation"),
                    ("manage_corps", "Can manage all Corporations"),
                    ("manage_own_alliance", "Can manage own Alliance"),
                    ("manage_alliances", "Can manage all Alliances"),
                ),
            },
        ),
        migrations.AlterField(
            model_name="allianceupdatestatus",
            name="section",
            field=models.CharField(
                choices=[
                    ("payment_system", "Payment System"),
                    ("payments", "Payments"),
                    ("payday", "Payday"),
                ],
                db_index=True,
                max_length=32,
            ),
        ),
        migrations.AlterField(
            model_name="corporationupdatestatus",
            name="section",
            field=models.CharField(
                choices=[
                    ("division_names", "Wallet Division Names"),
                    ("division", "Wallet Division"),
                    ("wallet", "Wallet Journal"),
                    ("members", "Members"),
                    ("payment_system", "Payment System"),
                    ("payments", "Payments"),
                    ("payday", "Payday"),
                ],
                db_index=True,
                max_length=32,
            ),
        ),
        migrations.RemoveField(
            model_name="corporationpayments",
            name="corporation_id",
        ),
        migrations.AddField(
            model_name="corporationpayments",
            name="owner_id",
            field=models.PositiveIntegerField(
                blank=True,
                help_text="ID of the owner (corporation or alliance) associated with this payment",
                null=True,
            ),
        ),
        migrations.AddIndex(
            model_name="alliancepayments",
            index=models.Index(
                fields=["account", "owner_id", "request_status", "-date"],
                name="taxsystem_a_account_677604_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="alliancepayments",
            index=models.Index(
                fields=["request_status", "-date"],
                name="taxsystem_a_request_d243c8_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="corporationpayments",
            index=models.Index(
                fields=["account", "owner_id", "request_status", "-date"],
                name="taxsystem_c_account_dafabb_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="corporationpayments",
            index=models.Index(
                fields=["request_status", "-date"],
                name="taxsystem_c_request_5fb377_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="members",
            index=models.Index(
                fields=["owner", "character_name"],
                name="taxsystem_m_owner_i_f6738f_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="members",
            index=models.Index(fields=["status"], name="taxsystem_m_status_38e108_idx"),
        ),
        migrations.AlterUniqueTogether(
            name="allianceupdatestatus",
            unique_together={("owner", "section")},
        ),
        migrations.AlterUniqueTogether(
            name="corporationupdatestatus",
            unique_together={("owner", "section")},
        ),
        migrations.AlterField(
            model_name="allianceowner",
            name="corporation",
            field=models.ForeignKey(
                help_text="The corporation that owns this alliance tax system.",
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="taxsystem.corporationowner",
            ),
        ),
        migrations.AlterModelOptions(
            name="general",
            options={
                "default_permissions": (),
                "managed": False,
                "permissions": (
                    ("basic_access", "Can access the Tax System"),
                    ("create_access", "Can add Corporation/Alliance"),
                    ("manage_own_corp", "Can manage own Corporation"),
                    ("manage_corps", "Can manage all Corporations"),
                    ("manage_own_alliance", "Can manage own Alliance"),
                    ("manage_alliances", "Can manage all Alliances"),
                ),
            },
        ),
        migrations.AlterField(
            model_name="allianceadminhistory",
            name="comment",
            field=models.TextField(
                blank=True,
                default="",
                help_text="Comment of the action",
                verbose_name="Comment",
            ),
        ),
        migrations.AlterField(
            model_name="allianceadminhistory",
            name="owner",
            field=models.ForeignKey(
                help_text="Owner that the action was performed on",
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_admin_history",
                to="taxsystem.allianceowner",
                verbose_name="Owner",
            ),
        ),
        migrations.AlterField(
            model_name="alliancefilter",
            name="value",
            field=models.CharField(max_length=255, unique=True),
        ),
        migrations.AlterField(
            model_name="allianceowner",
            name="active",
            field=models.BooleanField(
                default=True,
                help_text="Designates whether this alliance owner is active.",
            ),
        ),
        migrations.AlterField(
            model_name="allianceowner",
            name="tax_amount",
            field=models.DecimalField(
                decimal_places=0,
                default=0,
                help_text="Tax Amount in ISK that is set for the alliance. Max 16 Digits",
                max_digits=16,
                validators=[django.core.validators.MaxValueValidator(9999999999999999)],
            ),
        ),
        migrations.AlterField(
            model_name="allianceowner",
            name="tax_period",
            field=models.PositiveIntegerField(
                default=30,
                help_text="Tax Period in days for the alliance. Max 365 days. Default: 30 days",
                validators=[django.core.validators.MaxValueValidator(365)],
            ),
        ),
        migrations.AlterField(
            model_name="alliancepaymentaccount",
            name="owner",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_ally_tax_accounts",
                to="taxsystem.allianceowner",
            ),
        ),
        migrations.AlterField(
            model_name="alliancepaymenthistory",
            name="action",
            field=models.CharField(
                choices=[
                    ("", ""),
                    ("Status Changed", "Status Changed"),
                    ("Payment Added", "Payment Added"),
                    ("Custom Payment Added", "Custom Payment Added"),
                    ("Reviser Comment", "Reviser Comment"),
                ],
                default="",
                help_text="Action performed",
                max_length=20,
                verbose_name="Action",
            ),
        ),
        migrations.AlterField(
            model_name="alliancepaymenthistory",
            name="comment",
            field=models.TextField(
                blank=True,
                default="",
                help_text="Comment of the action",
                verbose_name="Comment",
            ),
        ),
        migrations.AlterField(
            model_name="alliancepaymenthistory",
            name="new_status",
            field=models.CharField(
                choices=[
                    ("approved", "Approved"),
                    ("pending", "Pending"),
                    ("rejected", "Rejected"),
                    ("needs_approval", "Requires Auditor"),
                ],
                help_text="New Status of the action",
                max_length=20,
                verbose_name="New Status",
            ),
        ),
        migrations.AlterField(
            model_name="corporationadminhistory",
            name="comment",
            field=models.TextField(
                blank=True,
                default="",
                help_text="Comment of the action",
                verbose_name="Comment",
            ),
        ),
        migrations.AlterField(
            model_name="corporationadminhistory",
            name="owner",
            field=models.ForeignKey(
                help_text="Owner that the action was performed on",
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_admin_history",
                to="taxsystem.corporationowner",
                verbose_name="Owner",
            ),
        ),
        migrations.AlterField(
            model_name="corporationfilter",
            name="value",
            field=models.CharField(max_length=255, unique=True),
        ),
        migrations.AlterField(
            model_name="corporationowner",
            name="active",
            field=models.BooleanField(
                default=True,
                help_text="Designates whether this corporation owner is active.",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpaymentaccount",
            name="owner",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corp_tax_accounts",
                to="taxsystem.corporationowner",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpaymenthistory",
            name="action",
            field=models.CharField(
                choices=[
                    ("", ""),
                    ("Status Changed", "Status Changed"),
                    ("Payment Added", "Payment Added"),
                    ("Custom Payment Added", "Custom Payment Added"),
                    ("Reviser Comment", "Reviser Comment"),
                ],
                default="",
                help_text="Action performed",
                max_length=20,
                verbose_name="Action",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpaymenthistory",
            name="comment",
            field=models.TextField(
                blank=True,
                default="",
                help_text="Comment of the action",
                verbose_name="Comment",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpaymenthistory",
            name="new_status",
            field=models.CharField(
                choices=[
                    ("approved", "Approved"),
                    ("pending", "Pending"),
                    ("rejected", "Rejected"),
                    ("needs_approval", "Requires Auditor"),
                ],
                help_text="New Status of the action",
                max_length=20,
                verbose_name="New Status",
            ),
        ),
        migrations.RemoveIndex(
            model_name="alliancepayments",
            name="taxsystem_a_account_677604_idx",
        ),
        migrations.RemoveIndex(
            model_name="alliancepayments",
            name="taxsystem_a_request_d243c8_idx",
        ),
        migrations.RemoveIndex(
            model_name="corporationpayments",
            name="taxsystem_c_account_dafabb_idx",
        ),
        migrations.RemoveIndex(
            model_name="corporationpayments",
            name="taxsystem_c_request_5fb377_idx",
        ),
        migrations.RemoveIndex(
            model_name="members",
            name="taxsystem_m_owner_i_f6738f_idx",
        ),
        migrations.RemoveIndex(
            model_name="members",
            name="taxsystem_m_status_38e108_idx",
        ),
        migrations.RemoveField(
            model_name="alliancepayments",
            name="owner_id",
        ),
        migrations.RemoveField(
            model_name="corporationpayments",
            name="owner_id",
        ),
        migrations.AddField(
            model_name="alliancepayments",
            name="journal",
            field=models.OneToOneField(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="taxsystem.corporationwalletjournalentry",
            ),
        ),
        migrations.AddField(
            model_name="alliancepayments",
            name="owner",
            field=models.ForeignKey(
                blank=True,
                help_text="Owner of this payment",
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_alliance_payments",
                to="taxsystem.allianceowner",
            ),
        ),
        migrations.AddField(
            model_name="corporationpayments",
            name="journal",
            field=models.OneToOneField(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="taxsystem.corporationwalletjournalentry",
            ),
        ),
        migrations.AddField(
            model_name="corporationpayments",
            name="owner",
            field=models.ForeignKey(
                blank=True,
                help_text="Owner of this payment",
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corporation_payments",
                to="taxsystem.corporationowner",
            ),
        ),
        migrations.AlterField(
            model_name="alliancepayments",
            name="account",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="taxsystem.alliancepaymentaccount",
            ),
        ),
        migrations.AlterField(
            model_name="allianceupdatestatus",
            name="section",
            field=models.CharField(
                choices=[
                    ("tax_accounts", "Tax Accounts"),
                    ("payments", "Payments"),
                    ("deadlines", "Deadlines"),
                ],
                db_index=True,
                max_length=32,
            ),
        ),
        migrations.AlterField(
            model_name="corporationpayments",
            name="account",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="taxsystem.corporationpaymentaccount",
            ),
        ),
        migrations.AlterField(
            model_name="corporationupdatestatus",
            name="section",
            field=models.CharField(
                choices=[
                    ("division_names", "Wallet Division Names"),
                    ("divisions", "Wallet Divisions"),
                    ("wallet", "Wallet Journal"),
                    ("members", "Members"),
                    ("tax_accounts", "Tax Accounts"),
                    ("payments", "Payments"),
                    ("deadlines", "Deadlines"),
                ],
                db_index=True,
                max_length=32,
            ),
        ),
        migrations.AlterField(
            model_name="alliancepaymenthistory",
            name="new_status",
            field=models.CharField(
                choices=[
                    ("approved", "approved"),
                    ("pending", "pending"),
                    ("rejected", "rejected"),
                    ("needs_approval", "requires auditor"),
                ],
                help_text="New Status of the action",
                max_length=20,
                verbose_name="New Status",
            ),
        ),
        migrations.AlterField(
            model_name="alliancepayments",
            name="request_status",
            field=models.CharField(
                choices=[
                    ("approved", "approved"),
                    ("pending", "pending"),
                    ("rejected", "rejected"),
                    ("needs_approval", "requires auditor"),
                ],
                default="pending",
                max_length=16,
                verbose_name="Request Status",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpaymenthistory",
            name="new_status",
            field=models.CharField(
                choices=[
                    ("approved", "approved"),
                    ("pending", "pending"),
                    ("rejected", "rejected"),
                    ("needs_approval", "requires auditor"),
                ],
                help_text="New Status of the action",
                max_length=20,
                verbose_name="New Status",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpayments",
            name="request_status",
            field=models.CharField(
                choices=[
                    ("approved", "approved"),
                    ("pending", "pending"),
                    ("rejected", "rejected"),
                    ("needs_approval", "requires auditor"),
                ],
                default="pending",
                max_length=16,
                verbose_name="Request Status",
            ),
        ),
        migrations.AlterField(
            model_name="allianceadminhistory",
            name="action",
            field=models.CharField(
                choices=[
                    ("", ""),
                    ("Added", "added"),
                    ("Changed", "changed"),
                    ("Deleted", "deleted"),
                ],
                default="",
                help_text="Action performed",
                max_length=20,
                verbose_name="Action",
            ),
        ),
        migrations.AlterField(
            model_name="corporationadminhistory",
            name="action",
            field=models.CharField(
                choices=[
                    ("", ""),
                    ("Added", "added"),
                    ("Changed", "changed"),
                    ("Deleted", "deleted"),
                ],
                default="",
                help_text="Action performed",
                max_length=20,
                verbose_name="Action",
            ),
        ),
        migrations.AlterField(
            model_name="alliancepaymentaccount",
            name="status",
            field=models.CharField(
                blank=True,
                choices=[
                    ("active", "active"),
                    ("inactive", "inactive"),
                    ("deactivated", "deactivated"),
                    ("missing", "missing"),
                ],
                default="active",
                max_length=16,
            ),
        ),
        migrations.AlterField(
            model_name="corporationpaymentaccount",
            name="status",
            field=models.CharField(
                blank=True,
                choices=[
                    ("active", "active"),
                    ("inactive", "inactive"),
                    ("deactivated", "deactivated"),
                    ("missing", "missing"),
                ],
                default="active",
                max_length=16,
            ),
        ),
        migrations.AlterField(
            model_name="members",
            name="status",
            field=models.CharField(
                blank=True,
                choices=[
                    ("active", "active"),
                    ("missing", "missing"),
                    ("noaccount", "unregistered"),
                    ("is_alt", "is alt"),
                ],
                default="active",
                max_length=10,
                verbose_name="Status",
            ),
        ),
        migrations.AddField(
            model_name="alliancefilter",
            name="match_type",
            field=models.CharField(
                choices=[("exact", "Exact Match"), ("contains", "Contains")],
                default="exact",
                max_length=20,
            ),
        ),
        migrations.AddField(
            model_name="corporationfilter",
            name="match_type",
            field=models.CharField(
                choices=[("exact", "Exact Match"), ("contains", "Contains")],
                default="exact",
                max_length=20,
            ),
        ),
        migrations.AlterField(
            model_name="allianceadminhistory",
            name="action",
            field=models.CharField(
                choices=[
                    ("", ""),
                    ("Added", "Added"),
                    ("Changed", "Changed"),
                    ("Deleted", "Deleted"),
                ],
                default="",
                help_text="Action performed",
                max_length=20,
                verbose_name="Action",
            ),
        ),
        migrations.AlterField(
            model_name="alliancepaymentaccount",
            name="status",
            field=models.CharField(
                blank=True,
                choices=[
                    ("active", "Active"),
                    ("inactive", "Inactive"),
                    ("deactivated", "Deactivated"),
                    ("missing", "Missing"),
                ],
                default="active",
                max_length=16,
            ),
        ),
        migrations.AlterField(
            model_name="alliancepaymenthistory",
            name="new_status",
            field=models.CharField(
                choices=[
                    ("approved", "Approved"),
                    ("pending", "Pending"),
                    ("rejected", "Rejected"),
                    ("needs_approval", "Requires Auditor"),
                ],
                help_text="New Status of the action",
                max_length=20,
                verbose_name="New Status",
            ),
        ),
        migrations.AlterField(
            model_name="alliancepayments",
            name="request_status",
            field=models.CharField(
                choices=[
                    ("approved", "Approved"),
                    ("pending", "Pending"),
                    ("rejected", "Rejected"),
                    ("needs_approval", "Requires Auditor"),
                ],
                default="pending",
                max_length=16,
                verbose_name="Request Status",
            ),
        ),
        migrations.AlterField(
            model_name="corporationadminhistory",
            name="action",
            field=models.CharField(
                choices=[
                    ("", ""),
                    ("Added", "Added"),
                    ("Changed", "Changed"),
                    ("Deleted", "Deleted"),
                ],
                default="",
                help_text="Action performed",
                max_length=20,
                verbose_name="Action",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpaymentaccount",
            name="status",
            field=models.CharField(
                blank=True,
                choices=[
                    ("active", "Active"),
                    ("inactive", "Inactive"),
                    ("deactivated", "Deactivated"),
                    ("missing", "Missing"),
                ],
                default="active",
                max_length=16,
            ),
        ),
        migrations.AlterField(
            model_name="corporationpaymenthistory",
            name="new_status",
            field=models.CharField(
                choices=[
                    ("approved", "Approved"),
                    ("pending", "Pending"),
                    ("rejected", "Rejected"),
                    ("needs_approval", "Requires Auditor"),
                ],
                help_text="New Status of the action",
                max_length=20,
                verbose_name="New Status",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpayments",
            name="request_status",
            field=models.CharField(
                choices=[
                    ("approved", "Approved"),
                    ("pending", "Pending"),
                    ("rejected", "Rejected"),
                    ("needs_approval", "Requires Auditor"),
                ],
                default="pending",
                max_length=16,
                verbose_name="Request Status",
            ),
        ),
        migrations.AlterField(
            model_name="members",
            name="status",
            field=models.CharField(
                blank=True,
                choices=[
                    ("active", "Active"),
                    ("missing", "Missing"),
                    ("noaccount", "Unregistered"),
                    ("is_alt", "is alt"),
                ],
                default="active",
                max_length=10,
                verbose_name="Status",
            ),
        ),
    ]
