# Generated by Django 4.2.26 on 2025-12-20 00:45

# Django
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("taxsystem", "0020_alter_general_options_and_more"),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name="alliancepayments",
            name="taxsystem_a_account_677604_idx",
        ),
        migrations.RemoveIndex(
            model_name="alliancepayments",
            name="taxsystem_a_request_d243c8_idx",
        ),
        migrations.RemoveIndex(
            model_name="corporationpayments",
            name="taxsystem_c_account_dafabb_idx",
        ),
        migrations.RemoveIndex(
            model_name="corporationpayments",
            name="taxsystem_c_request_5fb377_idx",
        ),
        migrations.RemoveIndex(
            model_name="members",
            name="taxsystem_m_owner_i_f6738f_idx",
        ),
        migrations.RemoveIndex(
            model_name="members",
            name="taxsystem_m_status_38e108_idx",
        ),
        migrations.RemoveField(
            model_name="alliancepayments",
            name="owner_id",
        ),
        migrations.RemoveField(
            model_name="corporationpayments",
            name="owner_id",
        ),
        migrations.AddField(
            model_name="alliancepayments",
            name="journal",
            field=models.OneToOneField(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="taxsystem.corporationwalletjournalentry",
            ),
        ),
        migrations.AddField(
            model_name="alliancepayments",
            name="owner",
            field=models.ForeignKey(
                blank=True,
                help_text="Owner of this payment",
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_alliance_payments",
                to="taxsystem.allianceowner",
            ),
        ),
        migrations.AddField(
            model_name="corporationpayments",
            name="journal",
            field=models.OneToOneField(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="taxsystem.corporationwalletjournalentry",
            ),
        ),
        migrations.AddField(
            model_name="corporationpayments",
            name="owner",
            field=models.ForeignKey(
                blank=True,
                help_text="Owner of this payment",
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corporation_payments",
                to="taxsystem.corporationowner",
            ),
        ),
        migrations.AlterField(
            model_name="alliancepayments",
            name="account",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="taxsystem.alliancepaymentaccount",
            ),
        ),
        migrations.AlterField(
            model_name="alliancepayments",
            name="entry_id",
            field=models.BigIntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name="allianceupdatestatus",
            name="section",
            field=models.CharField(
                choices=[
                    ("tax_accounts", "Tax Accounts"),
                    ("payments", "Payments"),
                    ("deadlines", "Deadlines"),
                ],
                db_index=True,
                max_length=32,
            ),
        ),
        migrations.AlterField(
            model_name="corporationpayments",
            name="account",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="taxsystem.corporationpaymentaccount",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpayments",
            name="entry_id",
            field=models.BigIntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name="corporationupdatestatus",
            name="section",
            field=models.CharField(
                choices=[
                    ("division_names", "Wallet Division Names"),
                    ("divisions", "Wallet Divisions"),
                    ("wallet", "Wallet Journal"),
                    ("members", "Members"),
                    ("tax_accounts", "Tax Accounts"),
                    ("payments", "Payments"),
                    ("deadlines", "Deadlines"),
                ],
                db_index=True,
                max_length=32,
            ),
        ),
    ]
