{% load i18n %}
{% load theme_tags %}

<!-- Bulk action info panel -->
<div id="tax-bulk-panel" class="card tax-bulk-actions col-sm-6 mb-3 position-fixed d-none fade" style="top: calc({% header_padding_size %} + 35rem);">
  <div class="card-body d-flex align-items-center">
    <div class="me-3">
      <strong id="tax-bulk-count">0</strong> {% translate "selected" %}
    </div>
    <div id="tax-bulk-buttons" class="btn-group">
      <button type="button" class="btn btn-sm btn-success"
        title="{% translate 'Activate selected accounts' %}"
        data-bulk-action="activate"
        data-bs-toggle="modal"
        data-bs-tooltip="aa-taxsystem"
        data-bs-target="#taxsystem-accept-bulk-actions"
      >{% translate "Activate Account" %}</button>
      <button type="button" class="btn btn-sm btn-warning"
        title="{% translate 'Deactivate selected tax accounts' %}"
        data-bulk-action="deactivate"
        data-bs-toggle="modal"
        data-bs-tooltip="aa-taxsystem"
        data-bs-target="#taxsystem-accept-bulk-actions"
      >{% translate "Deactivate Account" %}</button>
    </div>
    <button type="button" class="btn btn-danger btn-sm ms-2"
      id="tax-bulk-action-clear-selection"
      data-bs-tooltip="aa-taxsystem"
      title="{% translate 'Clear Selection' %}"
      >
        <i class="fa-solid fa-hand"></i> {% translate "Clear Selection" %}
    </button>
  </div>
</div>
