# Generated by Django 4.2.26 on 2025-11-14 13:34

# Django
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("taxsystem", "0010_create_alliance_corporation"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="corporationowner",
            options={"default_permissions": ()},
        ),
        migrations.AlterModelOptions(
            name="corporationpaymentaccount",
            options={"default_permissions": ()},
        ),
        migrations.AlterModelOptions(
            name="corporationpayments",
            options={"default_permissions": ()},
        ),
        migrations.RenameField(
            model_name="corporationowner",
            old_name="corporation",
            new_name="eve_corporation",
        ),
        migrations.RemoveField(
            model_name="corporationowner",
            name="alliance",
        ),
        migrations.AlterField(
            model_name="corporationpaymentaccount",
            name="owner",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="taxsystem.corporationowner",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpayments",
            name="account",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corp_payments",
                to="taxsystem.corporationpaymentaccount",
            ),
        ),
        migrations.AlterField(
            model_name="corporationupdatestatus",
            name="owner",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corporation_update_status",
                to="taxsystem.corporationowner",
            ),
        ),
        migrations.AlterField(
            model_name="journalfilter",
            name="filter_set",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corporation_filters",
                to="taxsystem.journalfilterset",
            ),
        ),
        migrations.AlterField(
            model_name="journalfilterset",
            name="owner",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corporation_filter_set",
                to="taxsystem.corporationowner",
            ),
        ),
    ]
