# Generated by Django 4.2.11 on 2025-02-16 15:37

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("taxsystem", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="paymenthistory",
            name="comment",
            field=models.CharField(
                choices=[
                    ("", ""),
                    ("Payment added to system", "Payment added to system"),
                    ("Automated approved Payment", "Automated approved Payment"),
                    (
                        "Payment must be approved by an reviser",
                        "Payment must be approved by an reviser",
                    ),
                ],
                default="",
                help_text="Comment of the action",
                max_length=128,
                verbose_name="Comment",
            ),
        ),
        migrations.AlterField(
            model_name="paymenthistory",
            name="new_status",
            field=models.CharField(
                choices=[
                    ("approved", "Approved"),
                    ("pending", "Pending"),
                    ("rejected", "Rejected"),
                    ("needs_approval", "Requires Auditor"),
                ],
                help_text="New Status of the action",
                max_length=16,
                verbose_name="New Status",
            ),
        ),
        migrations.AlterField(
            model_name="payments",
            name="request_status",
            field=models.CharField(
                choices=[
                    ("approved", "Approved"),
                    ("pending", "Pending"),
                    ("rejected", "Rejected"),
                    ("needs_approval", "Requires Auditor"),
                ],
                default="pending",
                max_length=16,
                verbose_name="Request Status",
            ),
        ),
    ]
