
{% load i18n %}

<table class="table w-100" id="filter-table">
    <thead>
        <tr>
            <th class="w-auto">{% translate "Filter Type" %}</th>
            <th class="w-auto">{% translate "Match Type" %}</th>
            <th class="w-auto">{% translate "Filter Value" %}</th>
            <th class="w-auto">{% translate "Actions" %}</th>
        </tr>
    </thead>
    <tbody class="modal-filter-table-body"></tbody>
</table>
