# Generated by Django 4.2.26 on 2025-12-03 20:17

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("taxsystem", "0018_alter_allianceowner_corporation"),
    ]

    operations = [
        migrations.AlterField(
            model_name="alliancepayments",
            name="entry_id",
            field=models.BigIntegerField(blank=True, null=True, unique=True),
        ),
        migrations.AlterField(
            model_name="corporationpayments",
            name="entry_id",
            field=models.BigIntegerField(blank=True, null=True, unique=True),
        ),
    ]
