{% load i18n %}

<div class="card-header bg-primary rounded-top d-flex align-items-center mt-4">
    <h3 class="me-3">{% translate "User" %}</h3>
</div>
<div class="card-body rounded-bottom">
    <!-- User Content-->
    <table class="table w-100 tax-table-hover" id="user">
        <thead>
            <tr>
                <th class="w-auto">{% translate "Portrait" %}</th>
                <th class="w-auto">{% translate "Account Name" %}</th>
                <th class="w-auto">{% translate "Status" %}</th>
                <th class="w-auto">{% translate "Joined" %}</th>
                <th class="w-auto">{% translate "Last Login" %}</th>
                <th class="w-auto">{% translate "Tax Paid" %}</th>
                <th class="w-auto">{% translate "Deposit" %}</th>
                <th class="w-auto">{% translate "Last Debit" %}</th>
            </tr>
        </thead>

        <tbody>
            <!-- User -->
        </tbody>
    </table>
</div>
