# Generated by Django 4.2.26 on 2025-12-16 12:47

# Django
import django.core.validators
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("taxsystem", "0019_alter_alliancepayments_entry_id_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="general",
            options={
                "default_permissions": (),
                "managed": False,
                "permissions": (
                    ("basic_access", "Can access the Tax System"),
                    ("create_access", "Can add Corporation/Alliance"),
                    ("manage_own_corp", "Can manage own Corporation"),
                    ("manage_corps", "Can manage all Corporations"),
                    ("manage_own_alliance", "Can manage own Alliance"),
                    ("manage_alliances", "Can manage all Alliances"),
                ),
            },
        ),
        migrations.AlterField(
            model_name="allianceadminhistory",
            name="comment",
            field=models.TextField(
                blank=True,
                default="",
                help_text="Comment of the action",
                verbose_name="Comment",
            ),
        ),
        migrations.AlterField(
            model_name="allianceadminhistory",
            name="owner",
            field=models.ForeignKey(
                help_text="Owner that the action was performed on",
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_admin_history",
                to="taxsystem.allianceowner",
                verbose_name="Owner",
            ),
        ),
        migrations.AlterField(
            model_name="alliancefilter",
            name="value",
            field=models.CharField(max_length=255, unique=True),
        ),
        migrations.AlterField(
            model_name="allianceowner",
            name="active",
            field=models.BooleanField(
                default=True,
                help_text="Designates whether this alliance owner is active.",
            ),
        ),
        migrations.AlterField(
            model_name="allianceowner",
            name="tax_amount",
            field=models.DecimalField(
                decimal_places=0,
                default=0,
                help_text="Tax Amount in ISK that is set for the alliance. Max 16 Digits",
                max_digits=16,
                validators=[django.core.validators.MaxValueValidator(9999999999999999)],
            ),
        ),
        migrations.AlterField(
            model_name="allianceowner",
            name="tax_period",
            field=models.PositiveIntegerField(
                default=30,
                help_text="Tax Period in days for the alliance. Max 365 days. Default: 30 days",
                validators=[django.core.validators.MaxValueValidator(365)],
            ),
        ),
        migrations.AlterField(
            model_name="alliancepaymentaccount",
            name="owner",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_ally_tax_accounts",
                to="taxsystem.allianceowner",
            ),
        ),
        migrations.AlterField(
            model_name="alliancepaymenthistory",
            name="action",
            field=models.CharField(
                choices=[
                    ("", ""),
                    ("Status Changed", "Status Changed"),
                    ("Payment Added", "Payment Added"),
                    ("Custom Payment Added", "Custom Payment Added"),
                    ("Reviser Comment", "Reviser Comment"),
                ],
                default="",
                help_text="Action performed",
                max_length=20,
                verbose_name="Action",
            ),
        ),
        migrations.AlterField(
            model_name="alliancepaymenthistory",
            name="comment",
            field=models.TextField(
                blank=True,
                default="",
                help_text="Comment of the action",
                verbose_name="Comment",
            ),
        ),
        migrations.AlterField(
            model_name="alliancepaymenthistory",
            name="new_status",
            field=models.CharField(
                choices=[
                    ("approved", "Approved"),
                    ("pending", "Pending"),
                    ("rejected", "Rejected"),
                    ("needs_approval", "Requires Auditor"),
                ],
                help_text="New Status of the action",
                max_length=20,
                verbose_name="New Status",
            ),
        ),
        migrations.AlterField(
            model_name="corporationadminhistory",
            name="comment",
            field=models.TextField(
                blank=True,
                default="",
                help_text="Comment of the action",
                verbose_name="Comment",
            ),
        ),
        migrations.AlterField(
            model_name="corporationadminhistory",
            name="owner",
            field=models.ForeignKey(
                help_text="Owner that the action was performed on",
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_admin_history",
                to="taxsystem.corporationowner",
                verbose_name="Owner",
            ),
        ),
        migrations.AlterField(
            model_name="corporationfilter",
            name="value",
            field=models.CharField(max_length=255, unique=True),
        ),
        migrations.AlterField(
            model_name="corporationowner",
            name="active",
            field=models.BooleanField(
                default=True,
                help_text="Designates whether this corporation owner is active.",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpaymentaccount",
            name="owner",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corp_tax_accounts",
                to="taxsystem.corporationowner",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpaymenthistory",
            name="action",
            field=models.CharField(
                choices=[
                    ("", ""),
                    ("Status Changed", "Status Changed"),
                    ("Payment Added", "Payment Added"),
                    ("Custom Payment Added", "Custom Payment Added"),
                    ("Reviser Comment", "Reviser Comment"),
                ],
                default="",
                help_text="Action performed",
                max_length=20,
                verbose_name="Action",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpaymenthistory",
            name="comment",
            field=models.TextField(
                blank=True,
                default="",
                help_text="Comment of the action",
                verbose_name="Comment",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpaymenthistory",
            name="new_status",
            field=models.CharField(
                choices=[
                    ("approved", "Approved"),
                    ("pending", "Pending"),
                    ("rejected", "Rejected"),
                    ("needs_approval", "Requires Auditor"),
                ],
                help_text="New Status of the action",
                max_length=20,
                verbose_name="New Status",
            ),
        ),
    ]
