{% load sri %}

{% sri_static 'taxsystem/css/aa-taxsystem.css' %}

<!-- Style -->
<style>
    @media (min-width: 768px) {
        .dl-horizontal dt {
            float: left;
            width: 140px;
            clear: left;
            text-align: left;
            overflow: hidden;
            text-overflow: ellipsis;
            margin-right: 8px;
        }
        .dl-horizontal dd {
            overflow: hidden;
            text-overflow: ellipsis;
        }
    }
</style>
