{% load sri %}

{% sri_static 'taxsystem/libs/x-editable/1.5.8/css/bootstrap-editable.css' %}
{% sri_static 'taxsystem/libs/x-editable/1.5.8/js/bootstrap-editable.js' %}
{% sri_static 'taxsystem/js/view-manage.js' %}
{% include 'bundles/moment-js.html' with locale=True %}
