# Generated by Django 4.2.28 on 2026-02-26 07:19

# Django
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("taxsystem", "0025_eveentity_and_more"),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name="corporationwalletjournalentry",
            name="taxsystem_c_first_p_d83db8_idx",
        ),
        migrations.RemoveIndex(
            model_name="corporationwalletjournalentry",
            name="taxsystem_c_second__2a759e_idx",
        ),
        migrations.RemoveField(
            model_name="corporationwalletjournalentry",
            name="first_party",
        ),
        migrations.RemoveField(
            model_name="corporationwalletjournalentry",
            name="second_party",
        ),
    ]
