# Generated by Django 4.2.26 on 2025-12-23 12:14

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("taxsystem", "0022_alter_alliancepaymenthistory_new_status_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="alliancefilter",
            name="match_type",
            field=models.CharField(
                choices=[("exact", "Exact Match"), ("contains", "Contains")],
                default="exact",
                max_length=20,
            ),
        ),
        migrations.AddField(
            model_name="corporationfilter",
            name="match_type",
            field=models.CharField(
                choices=[("exact", "Exact Match"), ("contains", "Contains")],
                default="exact",
                max_length=20,
            ),
        ),
    ]
