# Generated by Django 4.2.26 on 2025-11-19 21:26

# Django
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        (
            "taxsystem",
            "0014_rename_allianceownerupdatestatus_allianceupdatestatus_and_more",
        ),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="general",
            options={
                "default_permissions": (),
                "managed": False,
                "permissions": (
                    ("basic_access", "Can access the Tax System"),
                    ("create_access", "Can add Corporation"),
                    ("manage_own_corp", "Can manage own Corporation"),
                    ("manage_corps", "Can manage all Corporations"),
                    ("manage_own_alliance", "Can manage own Alliance"),
                    ("manage_alliances", "Can manage all Alliances"),
                ),
            },
        ),
    ]
