# Generated by Django 4.2.23 on 2025-07-22 15:48

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("taxsystem", "0006_create_update_status_existings"),
    ]

    operations = [
        migrations.AlterField(
            model_name="ownerupdatestatus",
            name="section",
            field=models.CharField(
                choices=[
                    ("wallet", "Wallet Journal"),
                    ("division_names", "Wallet Division Names"),
                    ("division", "Wallet Division"),
                    ("members", "Members"),
                    ("payments", "Payments"),
                    ("payment_system", "Payment System"),
                    ("payday", "Payday"),
                ],
                db_index=True,
                max_length=32,
            ),
        ),
    ]
