{% extends "taxsystem/base.html" %}
{% load i18n %}

{% block page_title %}
    {{ title }} ⋗ {% translate "Tax System" %}
{% endblock %}

{% block page_topic %}
    <h1 class="page-header text-center">
        {{ title }}
    </h1>
{% endblock page_topic %}

{% block header_nav_collapse_left %}

{% include 'taxsystem/partials/navigation/navigation.html' %}

{% endblock header_nav_collapse_left %}

{% block header_nav_collapse_right %}

{% include 'taxsystem/partials/navigation/administration.html' %}

{% endblock header_nav_collapse_right %}

{% block taxsystem_block %}
    <div class="card-body">
        <div class="card">
            <div class="card-header text-center bg-primary">{% translate "Administration" %}</div>
            <div class="card-body w-25">
                <form action="{% url 'taxsystem:admin' %}" method="post">
                    {% csrf_token %}
                    <div class="mb-3 form-check">
                        <input class="form-check-input" type="checkbox" name="force_refresh" id="force_refresh">
                        <label class="form-check-label" for="force_refresh">{% translate "Mark all tasks as Force Refresh" %}</label>
                    </div>
                    <hr class="my-4"> <!-- Bootstrap Trennlinie -->
                    <div class="mb-3 form-check">
                        <input class="form-check-input" type="checkbox" name="run_clear_etag" id="run_clear_etag">
                        <label class="form-check-label" for="run_clear_etag">{% translate "Clear all cached ETags" %}</label>
                    </div>
                    <div class="mb-3 form-check">
                        <input class="form-check-input" type="checkbox" name="run_taxsystem_updates" id="run_taxsystem_updates">
                        <label class="form-check-label" for="run_taxsystem_updates">{% translate "All Taxsystem Updates" %}</label>
                    </div>
                    <div class="mb-3 form-check">
                        <input class="form-check-input" type="checkbox" name="run_taxsystem_corporation_updates" id="run_taxsystem_corporation_updates">
                        <label class="form-check-label" for="run_taxsystem_corporation_updates">{% translate "Update Taxsystem Corporation Updates" %}</label>
                    </div>
                    <div class="mb-3">
                        <label for="corporation_id" class="form-label">{% translate "Corporation ID (optional)" %}</label>
                        <input type="number" class="form-control" name="corporation_id" id="corporation_id" placeholder="{% translate 'Enter Corporation ID' %}">
                        <small class="form-text text-muted">{% translate "Leave empty to update all corporations" %}</small>
                    </div>
                    <div class="mb-3 form-check">
                        <input class="form-check-input" type="checkbox" name="run_taxsystem_alliance_updates" id="run_taxsystem_alliance_updates">
                        <label class="form-check-label" for="run_taxsystem_alliance_updates">{% translate "Update Taxsystem Alliance Updates" %}</label>
                    </div>
                    <div class="mb-3">
                        <label for="alliance_id" class="form-label">{% translate "Alliance ID (optional)" %}</label>
                        <input type="number" class="form-control" name="alliance_id" id="alliance_id" placeholder="{% translate 'Enter Alliance ID' %}">
                        <small class="form-text text-muted">{% translate "Leave empty to update all alliances" %}</small>
                    </div>
                    <input type="submit" value="{% translate 'Submit' %}" class="btn col-md-12 btn-primary" />
                </form>
            </div>
        </div>
    </div>
{% endblock taxsystem_block %}
