# Generated by Django 4.2.26 on 2025-11-22 16:46

# Django
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("taxsystem", "0016_remove_corporationpayments_corporation_id_and_more"),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name="allianceupdatestatus",
            unique_together={("owner", "section")},
        ),
        migrations.AlterUniqueTogether(
            name="corporationupdatestatus",
            unique_together={("owner", "section")},
        ),
    ]
