# Generated by Django 4.2.11 on 2025-02-16 20:29

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("taxsystem", "0002_alter_paymenthistory_comment_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="general",
            options={
                "default_permissions": (),
                "managed": False,
                "permissions": (
                    ("basic_access", "Can access the Tax System"),
                    ("create_access", "Can add Corporation"),
                    ("manage_own_corp", "Can manage own Corporation"),
                    ("manage_corps", "Can manage all Corporations"),
                ),
            },
        ),
        migrations.AlterField(
            model_name="paymenthistory",
            name="comment",
            field=models.CharField(
                choices=[
                    ("", ""),
                    ("Payment added to system", "Payment added to system"),
                    ("Automated approved Payment", "Automated approved Payment"),
                    (
                        "Payment must be approved by an reviser",
                        "Payment must be approved by an reviser",
                    ),
                ],
                default="",
                help_text="Comment of the action",
                max_length=255,
                verbose_name="Comment",
            ),
        ),
    ]
