# Generated by Django 4.2.26 on 2025-11-17 11:49

# Django
import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("taxsystem", "0012_alliancefilter_alliancefilterset_and_more"),
    ]

    operations = [
        migrations.RenameModel(
            old_name="AdminLogs",
            new_name="CorporationAdminHistory",
        ),
        migrations.RenameModel(
            old_name="PaymentHistory",
            new_name="CorporationPaymentHistory",
        ),
        migrations.AlterField(
            model_name="corporationpaymentaccount",
            name="owner",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corporation_payment_accounts",
                to="taxsystem.corporationowner",
            ),
        ),
        migrations.AlterField(
            model_name="corporationpayments",
            name="account",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="ts_corporation_payments",
                to="taxsystem.corporationpaymentaccount",
            ),
        ),
        migrations.AlterField(
            model_name="corporationupdatestatus",
            name="section",
            field=models.CharField(
                choices=[
                    ("wallet", "Wallet Journal"),
                    ("division_names", "Wallet Division Names"),
                    ("division", "Wallet Division"),
                    ("members", "Members"),
                    ("payments", "Payments"),
                    ("payment_system", "Payment System"),
                    ("payday", "Payday"),
                ],
                db_index=True,
                max_length=32,
            ),
        ),
    ]
