{% load i18n %}
<div class="table-responsive">
    <div class="float-end">
        {% translate "Filter by Payment Status" %}

        <div class="btn-group btn-group-aa-taxsystem ms-2" title="{% translate 'Filter by Payment Status' %}">
            <input type="radio" class="btn-check" id="request-filter-all" name="request-filter" autocomplete="off" checked>
            <label for="request-filter-all" class="btn btn-outline-primary">
                {% translate "All" %}
            </label>
            <input type="radio" class="btn-check" id="request-filter-pending" name="request-filter" autocomplete="off">
            <label for="request-filter-pending" class="btn btn-outline-primary">
                {% translate "Pending" %}
            </label>
        </div>
    </div>
    <!-- Payments Content-->
    <table class="table w-100 tax-table-hover" id="payments">
        <thead>
            <tr>
                <th class="w-auto">{% translate "Portrait" %}</th>
                <th class="w-auto">{% translate "Character Name" %}</th>
                <th class="w-auto">{% translate "ISK" %}</th>
                <th class="w-auto">{% translate "Date" %}</th>
                <th class="w-auto">{% translate "Status" %}</th>
                <th class="w-auto">{% translate "Actions" %}</th>
            </tr>
        </thead>

        <tbody>
            <!-- Payment Row -->
        </tbody>
    </table>
</div>
