table.dataTable.fixedHeader-floating,
table.dataTable.fixedHeader-locked {
  position: relative !important;
  background-color: var(--bs-body-bg);
  margin-top: 0 !important;
  margin-bottom: 0 !important;
}

div.dtfh-floatingparent-foot table {
  border-top-color: var(--bs-border-color);
  border-top-width: var(--bs-border-width);
  border-top-style: solid;
}

@media print {
  table.fixedHeader-floating,
  table.fixedHeader-locked {
    display: none;
  }
}
