# Generated by Django 4.2.20 on 2025-05-04 21:51

# Django
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        (
            "taxsystem",
            "0005_rename_division_corporationwalletdivision_division_id_and_more",
        ),
    ]

    operations = []
