# Generated by Django 4.2.28 on 2026-02-26 07:20

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "taxsystem",
            "0026_remove_corporationwalletjournalentry_taxsystem_c_first_p_d83db8_idx_and_more",
        ),
    ]

    operations = [
        migrations.RenameField(
            model_name="corporationwalletjournalentry",
            old_name="first_party_new",
            new_name="first_party",
        ),
        migrations.RenameField(
            model_name="corporationwalletjournalentry",
            old_name="second_party_new",
            new_name="second_party",
        ),
        migrations.AddIndex(
            model_name="corporationwalletjournalentry",
            index=models.Index(
                fields=["first_party"], name="taxsystem_c_first_p_d83db8_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="corporationwalletjournalentry",
            index=models.Index(
                fields=["second_party"], name="taxsystem_c_second__2a759e_idx"
            ),
        ),
    ]
