{% load i18n %}
<div class="table-responsive">
    <!-- Payments Content-->
    <table class="table w-100 tax-table-hover" id="filter-sets-table">
        <thead>
            <tr>
                <th class="w-auto">{% translate "Name" %}</th>
                <th class="w-auto">{% translate "Descriptions" %}</th>
                <th class="w-auto">{% translate "Active" %}</th>
                <th class="w-auto">{% translate "Actions" %}</th>
            </tr>
        </thead>
        <tbody></tbody>
    </table>
</div>
